/**
 * The MIT License
 *
 * Copyright (c) 2010-2016 Google, Inc. http://angularjs.org
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('rxjs'), require('@angular/forms'), require('rxjs/operators'), require('@angular/common')) :
	typeof define === 'function' && define.amd ? define('nthm', ['exports', '@angular/core', 'rxjs', '@angular/forms', 'rxjs/operators', '@angular/common'], factory) :
	(factory((global.nthm = {}),global.ng.core,global.rxjs,global.ng.forms,global.Rx.Observable.prototype,global.ng.common));
}(this, (function (exports,core,rxjs,forms,operators,common) { 'use strict';

var Key = {
    Backspace: 8,
    Tab: 9,
    Enter: 13,
    Shift: 16,
    Escape: 27,
    Space: 32,
    End: 35,
    Home: 36,
    ArrowLeft: 37,
    ArrowUp: 38,
    ArrowRight: 39,
    ArrowDown: 40,
};
Key[Key.Backspace] = "Backspace";
Key[Key.Tab] = "Tab";
Key[Key.Enter] = "Enter";
Key[Key.Shift] = "Shift";
Key[Key.Escape] = "Escape";
Key[Key.Space] = "Space";
Key[Key.End] = "End";
Key[Key.Home] = "Home";
Key[Key.ArrowLeft] = "ArrowLeft";
Key[Key.ArrowUp] = "ArrowUp";
Key[Key.ArrowRight] = "ArrowRight";
Key[Key.ArrowDown] = "ArrowDown";
var styleProperties = Object.freeze([
    'direction',
    'boxSizing',
    'width',
    'height', 'overflowX',
    'overflowY',
    'borderTopWidth', 'borderRightWidth', 'borderBottomWidth', 'borderLeftWidth', 'borderStyle',
    'paddingTop', 'paddingRight', 'paddingBottom', 'paddingLeft',
    'fontStyle', 'fontVariant', 'fontWeight', 'fontStretch', 'fontSize', 'fontSizeAdjust', 'lineHeight', 'fontFamily',
    'textAlign', 'textTransform', 'textIndent',
    'textDecoration',
    'letterSpacing', 'wordSpacing',
    'tabSize', 'MozTabSize'
]);
var isBrowser = typeof window !== 'undefined';
var isFirefox = isBrowser && ((window)).mozInnerScreenX != null;
function isInputOrTextAreaElement(element) {
    return element !== null &&
        (element.nodeName === 'INPUT' || element.nodeName === 'TEXTAREA' || element.nodeName === '#text');
}
function spliceString(value, start, end, insert) {
    return value.substring(0, start) + insert + value.substring(end) + '';
}
function iterateMentionsMarkup(value, mentionMarkup, textIterator, markupIterator, displayTransform) {
    var match, start = 0, currentPlainTextIndex = 0;
    var regEx = mentionMarkup.regEx;
    regEx.lastIndex = 0;
    while ((match = regEx.exec(value)) !== null) {
        var display = displayTransform.apply(void 0, match);
        var substr = value.substring(start, match.index);
        textIterator(substr, start, currentPlainTextIndex);
        currentPlainTextIndex += substr.length;
        markupIterator(match[0], match.index, currentPlainTextIndex, display);
        currentPlainTextIndex += display.length;
        start = regEx.lastIndex;
    }
    if (start < value.length) {
        textIterator(value.substring(start), start, currentPlainTextIndex);
    }
}
function iterateOnlyMentionsMarkup(value, mentionMarkup, markupIterator, displayTransform) {
    var match, start = 0, currentPlainTextIndex = 0;
    var regEx = mentionMarkup.regEx;
    regEx.lastIndex = 0;
    while ((match = regEx.exec(value)) !== null) {
        var display = displayTransform.apply(void 0, match);
        var substr = value.substring(start, match.index);
        currentPlainTextIndex += substr.length;
        markupIterator(match[0], match.index, currentPlainTextIndex, display);
        currentPlainTextIndex += display.length;
        start = regEx.lastIndex;
    }
}
function mapPlainTextIndex(value, mentionMarkup, indexInPlainText, toEndOfMarkup, displayTransform) {
    if (isNaN(indexInPlainText)) {
        return indexInPlainText;
    }
    var result;
    var textIterator = function (matchString, index, substringPlainTextIndex) {
        if (typeof result !== 'undefined') {
            return;
        }
        if (substringPlainTextIndex + matchString.length >= indexInPlainText) {
            result = index + indexInPlainText - substringPlainTextIndex;
        }
    };
    var markupIterator = function (matchString, index, mentionPlainTextIndex, display) {
        if (typeof result !== 'undefined') {
            return;
        }
        if (mentionPlainTextIndex + display.length > indexInPlainText) {
            result = index + (toEndOfMarkup ? matchString.length : 0);
        }
    };
    iterateMentionsMarkup(value, mentionMarkup, textIterator, markupIterator, displayTransform);
    return typeof result !== 'undefined' ? result : value.length;
}
function getCaretPosition(element) {
    if (isInputOrTextAreaElement(element)) {
        var value = element.value;
        return value.slice(0, element.selectionStart).length;
    }
    var selection = window.getSelection();
    if (selection.rangeCount > 0) {
        var range = selection.getRangeAt(0);
        return range.startOffset;
    }
    return 0;
}
function getCaretCoordinates(element, position) {
    var coords = { top: 0, left: 0 };
    if (!isBrowser) {
        return coords;
    }
    var div = document.createElement('div');
    document.body.appendChild(div);
    var style = div.style;
    var computed = window.getComputedStyle ? getComputedStyle(element) : ((element)).currentStyle;
    style.whiteSpace = 'pre-wrap';
    if (element.nodeName !== 'INPUT') {
        style.wordWrap = 'break-word';
    }
    style.position = 'absolute';
    style.visibility = 'hidden';
    styleProperties.forEach(function (prop) { return style[prop] = computed[prop]; });
    if (isFirefox) {
        if (element.scrollHeight > parseInt(computed.height, 10)) {
            style.overflowY = 'scroll';
        }
    }
    else {
        style.overflow = 'hidden';
    }
    div.textContent = element.value.substring(0, position);
    if (element.nodeName === 'INPUT') {
        div.textContent = div.textContent.replace(/\s/g, '\u00a0');
    }
    var span = document.createElement('span');
    span.textContent = element.value.substring(position) || '.';
    div.appendChild(span);
    coords = {
        top: span.offsetTop + parseInt(computed['borderTopWidth'], 10),
        left: span.offsetLeft + parseInt(computed['borderLeftWidth'], 10)
    };
    document.body.removeChild(div);
    return coords;
}
function setCaretPosition(element, position) {
    if (isInputOrTextAreaElement(element) && element.selectionStart !== undefined) {
        element.focus();
        element.setSelectionRange(position, position);
    }
    else {
        var range = document.createRange();
        range.setStart(element, position);
        range.collapse(true);
        var selection = window.getSelection();
        selection.removeAllRanges();
        selection.addRange(range);
    }
}
function escapeRegExp(str) {
    return str.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
}
function markupToRegExp(markup) {
    var markupPattern = escapeRegExp(markup);
    var placeholderRegExp = /__([\w]+)__/g;
    var placeholders = {};
    var groups = /[\[\]()]+/.test(markup);
    var match, i = groups ? 1 : 0, placeholderExclusion = groups ? '^\\)\\]' : '\\S';
    do {
        match = placeholderRegExp.exec(markupPattern);
        if (match) {
            var placeholder = match[1];
            markupPattern = markupPattern.replace("__" + placeholder + "__", "([" + placeholderExclusion + "]+)");
            placeholders[placeholder] = ++i;
        }
    } while (match);
    return { markup: markup, regEx: new RegExp('(' + markupPattern + ')', 'ig'), groups: placeholders };
}
function getPlainText(value, mentionMarkup, displayTransform) {
    mentionMarkup.regEx.lastIndex = 0;
    return value.replace(mentionMarkup.regEx, displayTransform);
}
function replacePlaceholders(item, markupMention) {
    var result = markupMention.markup + '';
    Object.keys(markupMention.groups)
        .forEach(function (key) { return result = result.replace(new RegExp("__" + key + "__", 'g'), typeof item === 'string' ? item : item[key]); });
    return result;
}
function applyChangeToValue(value, markupMention, plainTextValue, selectionStartBeforeChange, selectionEndBeforeChange, selectionEndAfterChange, displayTransform) {
    if (selectionStartBeforeChange === void 0) {
        selectionStartBeforeChange = 0;
    }
    if (selectionEndBeforeChange === void 0) {
        selectionEndBeforeChange = 0;
    }
    if (selectionEndAfterChange === void 0) {
        selectionEndAfterChange = 0;
    }
    var oldPlainTextValue = getPlainText(value, markupMention, displayTransform);
    var lengthDelta = oldPlainTextValue.length - plainTextValue.length;
    if (!selectionStartBeforeChange) {
        selectionStartBeforeChange = selectionEndBeforeChange + lengthDelta;
    }
    if (!selectionEndBeforeChange) {
        selectionEndBeforeChange = selectionStartBeforeChange;
    }
    if (selectionStartBeforeChange === selectionEndBeforeChange && selectionEndBeforeChange === selectionEndAfterChange &&
        oldPlainTextValue.length === plainTextValue.length) {
        selectionStartBeforeChange--;
    }
    var insert = plainTextValue.slice(selectionStartBeforeChange, selectionEndAfterChange);
    var spliceStart = Math.min(selectionStartBeforeChange, selectionEndAfterChange);
    var spliceEnd = selectionEndBeforeChange;
    if (selectionStartBeforeChange === selectionEndAfterChange) {
        spliceEnd = Math.max(selectionEndBeforeChange, selectionStartBeforeChange + lengthDelta);
    }
    return spliceString(value, mapPlainTextIndex(value, markupMention, spliceStart, false, displayTransform), mapPlainTextIndex(value, markupMention, spliceEnd, true, displayTransform), insert);
}
function findStartOfMentionInPlainText(value, mentionMarkup, indexInPlainText, displayTransform) {
    var result = { start: -1, end: -1 };
    var markupIterator = function (matchString, index, mentionPlainTextIndex, display) {
        if (mentionPlainTextIndex < indexInPlainText && mentionPlainTextIndex + display.length > indexInPlainText) {
            result = { start: mentionPlainTextIndex, end: mentionPlainTextIndex + display.length };
            return true;
        }
        return false;
    };
    iterateOnlyMentionsMarkup(value, mentionMarkup, markupIterator, displayTransform);
    return result;
}
function getBoundsOfMentionAtPosition(value, mentionMarkup, indexInPlainText, displayTransform) {
    return findStartOfMentionInPlainText(value, mentionMarkup, indexInPlainText, displayTransform);
}
var NgMentionsListComponent = (function () {
    function NgMentionsListComponent() {
        this.activeIndex = -1;
        this.itemSelected = new core.EventEmitter();
        this.show = false;
        this.dropUp = false;
        this._top = 0;
        this._left = 0;
    }
    Object.defineProperty(NgMentionsListComponent.prototype, "selectedItem", {
        get: function () {
            return this.activeIndex >= 0 && this.items[this.activeIndex] !== undefined ? this.items[this.activeIndex] : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "top", {
        get: function () {
            return this._top + this.adjustTop + 'px';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "left", {
        get: function () {
            return this._left + 'px';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "noItems", {
        get: function () {
            return !Array.isArray(this.items) || this.items.length === 0;
        },
        enumerable: true,
        configurable: true
    });
    NgMentionsListComponent.prototype.ngOnInit =
        function () {
            if (!this.itemTemplate) {
                this.itemTemplate = this.defaultItemTemplate;
            }
        };
    NgMentionsListComponent.prototype.onItemClick =
        function (event, activeIndex, item) {
            event.preventDefault();
            this.activeIndex = activeIndex;
            this.itemSelected.emit(item);
        };
    NgMentionsListComponent.prototype.selectFirstItem =
        function () {
            this.activeIndex = 0;
            this.resetScroll();
        };
    NgMentionsListComponent.prototype.selectPreviousItem =
        function () {
            if (this.activeIndex > 0) {
                this.activeIndex--;
            }
            this.scrollToActiveItem();
        };
    NgMentionsListComponent.prototype.selectNextItem =
        function () {
            if (this.activeIndex < this.items.length - 1) {
                this.activeIndex++;
                this.scrollToActiveItem();
            }
        };
    NgMentionsListComponent.prototype.selectLastItem =
        function () {
            this.activeIndex = this.items.length > 0 ? this.items.length - 1 : 0;
            this.scrollToActiveItem();
        };
    NgMentionsListComponent.prototype.position =
        function () {
            var element = this.textAreaElement;
            var coords = getCaretCoordinates(element, element.selectionStart);
            this._top = coords.top;
            this._left = coords.left + element.offsetLeft;
            this.list.nativeElement.scrollTop = 0;
        };
    NgMentionsListComponent.prototype.resetScroll =
        function () {
            this.list.nativeElement.scrollTop = 0;
        };
    NgMentionsListComponent.prototype.transformItem =
        function (item) {
            return this.displayTransform(item) || item;
        };
    Object.defineProperty(NgMentionsListComponent.prototype, "adjustTop", {
        get: function () {
            var adjust = 0;
            if (!this.dropUp) {
                var computed = getComputedStyle(this.textAreaElement);
                adjust = parseInt(computed.fontSize, 10) + this.textAreaElement.offsetTop;
            }
            return adjust;
        },
        enumerable: true,
        configurable: true
    });
    NgMentionsListComponent.prototype.scrollToActiveItem =
        function () {
            var element = (this.list.nativeElement);
            if (this.activeIndex === 0) {
                element.scrollTop = 0;
            }
            else {
                var activeElement = (element.querySelector('li.active'));
                if (activeElement) {
                    element.scrollTop = activeElement.offsetTop;
                }
            }
        };
    NgMentionsListComponent.decorators = [
        { type: core.Component, args: [{
                    selector: 'mentions-list',
                    template: "\n      <ng-template #defaultItemTemplate let-item=\"item\">\n          {{transformItem(item)}}\n      </ng-template>\n      <ul #list class=\"dropdown-menu scrollable-menu\">\n          <li *ngFor=\"let item of items; let i = index\" [class.active]=\"activeIndex === i\">\n              <a href class=\"dropdown-item\" (click)=\"onItemClick($event, i, item)\">\n                  <ng-template [ngTemplateOutlet]=\"itemTemplate\"\n                               [ngTemplateOutletContext]=\"{item:item,index:i}\"></ng-template>\n              </a>\n          </li>\n      </ul>\n  ",
                    styles: [
                        'mentions-list {position: absolute;display: none;}', 'mentions-list.drop-up {transform: translateY(-100%);}',
                        'mentions-list.show {display: block;} mentions-list.no-items {display: none;}',
                        'mentions-list .scrollable-menu {display: block;height: auto;max-height:300px;overflow:auto;}',
                        'mentions-list li.active {background: #f7f7f9;}'
                    ],
                    encapsulation: core.ViewEncapsulation.None
                },] },
    ];
    NgMentionsListComponent.ctorParameters = function () { return []; };
    NgMentionsListComponent.propDecorators = {
        "defaultItemTemplate": [{ type: core.ViewChild, args: ['defaultItemTemplate',] },],
        "list": [{ type: core.ViewChild, args: ['list',] },],
        "show": [{ type: core.HostBinding, args: ['class.show',] },],
        "dropUp": [{ type: core.HostBinding, args: ['class.drop-up',] },],
        "top": [{ type: core.HostBinding, args: ['style.top',] },],
        "left": [{ type: core.HostBinding, args: ['style.left',] },],
        "noItems": [{ type: core.HostBinding, args: ['class.no-items',] },],
    };
    return NgMentionsListComponent;
}());
var NgMentionsComponent = (function () {
    function NgMentionsComponent(element, componentResolver, viewContainer, changeDetector, ngZone) {
        this.element = element;
        this.componentResolver = componentResolver;
        this.viewContainer = viewContainer;
        this.changeDetector = changeDetector;
        this.ngZone = ngZone;
        this.triggerChar = '@';
        this.mentionMarkup = '@[__display__](__type__:__id__)';
        this.disableSearch = false;
        this.maxItems = -1;
        this.dropUp = false;
        this.displayName = 'display';
        this.search = new core.EventEmitter();
        this.valueChanges = new core.EventEmitter();
        this.stateChanges = new rxjs.Subject();
        this.displayContent = '';
        this.lines = [];
        this.highlighterStyle = {};
        this.textAreaClassNames = {};
        this.mentions = [];
        this._value = '';
        this._rows = 1;
        this._columns = 1;
        this.stopSearch = false;
        this._destroyed = new rxjs.Subject();
        this.newLine = /\n/g;
        this._errorState = false;
    }
    Object.defineProperty(NgMentionsComponent.prototype, "formClass", {
        get: function () {
            return Object.keys(this.textAreaClassNames).join(' ');
        },
        set: function (classNames) {
            var _this = this;
            this.textAreaClassNames = {};
            Array.from(classNames.split(' ')).forEach(function (className) {
                _this.textAreaClassNames[className] = true;
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "value", {
        get: function () {
            return this._value;
        },
        set: function (value) {
            this.parseLines(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "required", {
        get: function () {
            return this._required;
        },
        set: function (value) {
            this._required = value;
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "disabled", {
        get: function () {
            return this._disabled;
        },
        set: function (value) {
            this._disabled = value;
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "rows", {
        get: function () {
            return this._rows;
        },
        set: function (value) {
            this._rows = Math.max(1, (value));
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "columns", {
        get: function () {
            return this._columns;
        },
        set: function (value) {
            this._columns = Math.max(1, (value));
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "mentionItems", {
        set: function (value) {
            this.mentions = value;
            if (this.disableSearch && this.mentionsList) {
                this.mentionsList.items = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "readonly", {
        get: function () {
            return this.disabled ? 'readonly' : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "errorState", {
        get: function () {
            return this._errorState;
        },
        enumerable: true,
        configurable: true
    });
    NgMentionsComponent.prototype.ngOnInit =
        function () {
            this.parseMarkup();
        };
    NgMentionsComponent.prototype.ngOnChanges =
        function (changes) {
            if ('markup' in changes) {
                this.parseMarkup();
            }
        };
    NgMentionsComponent.prototype.ngAfterViewInit =
        function () {
            this.parseLines(this._value);
            this.refreshStyles();
        };
    NgMentionsComponent.prototype.ngAfterViewChecked =
        function () {
            this.refreshStyles();
        };
    NgMentionsComponent.prototype.ngOnDestroy =
        function () {
            this._destroyed.next();
            this._destroyed.complete();
        };
    NgMentionsComponent.prototype.onWindowResize =
        function () {
            this.refreshStyles();
        };
    NgMentionsComponent.prototype.onTextAreaScroll =
        function () {
            this.highlighterElement.nativeElement.scrollTop = this.textAreaInputElement.nativeElement.scrollTop;
        };
    NgMentionsComponent.prototype.onSelect =
        function (event) {
            this.selectionStart = event.target.selectionStart;
            this.selectionEnd = event.target.selectionEnd;
        };
    NgMentionsComponent.prototype.onChange =
        function (newPlainTextValue) {
            var _this = this;
            var value = this._value;
            var displayTransform = this.displayTransform.bind(this);
            var selectionStart = this.textAreaInputElement.nativeElement.selectionStart;
            var selectionEnd = this.textAreaInputElement.nativeElement.selectionEnd;
            var bounds = getBoundsOfMentionAtPosition(newPlainTextValue, this.markupSearch, selectionStart, displayTransform);
            if (bounds.start !== -1) {
                newPlainTextValue = newPlainTextValue.substring(0, bounds.start) + newPlainTextValue.substring(bounds.end);
            }
            var newValue = applyChangeToValue(value, this.markupSearch, newPlainTextValue, this.selectionStart, this.selectionEnd, selectionEnd, displayTransform);
            var startOfMention = findStartOfMentionInPlainText(value, this.markupSearch, selectionStart, displayTransform);
            if (startOfMention.start > -1 && this.selectionEnd > startOfMention.start) {
                selectionStart = startOfMention.start;
                selectionEnd = selectionStart;
            }
            this.selectionStart = Math.max(selectionStart, 0);
            this.selectionEnd = Math.max(selectionEnd, 0);
            this.parseLines(newValue);
            if (this.selectionEnd > 0) {
                setTimeout(function () { return setCaretPosition(_this.textAreaInputElement.nativeElement, _this.selectionEnd); });
            }
        };
    NgMentionsComponent.prototype.onKeyDown =
        function (event) {
            var caretPosition = getCaretPosition(this.textAreaInputElement.nativeElement);
            var characterPressed = event.key;
            var keyCode = event.which || event.keyCode;
            if (!characterPressed) {
                var characterCode = event.which || event.keyCode;
                characterPressed = String.fromCharCode(characterCode);
                if (!event.shiftKey && (characterCode >= 65 && characterCode <= 90)) {
                    characterPressed = String.fromCharCode(characterCode + 32);
                }
            }
            if (keyCode === Key.Enter && event.wasSelection && caretPosition < this.startPos) {
                caretPosition = this.startNode.length;
                setCaretPosition(this.startNode, caretPosition);
            }
            var startOfMention = findStartOfMentionInPlainText(this._value, this.markupSearch, caretPosition, this.displayTransform.bind(this));
            if (characterPressed === this.triggerChar) {
                this.setupMentionsList(caretPosition);
            }
            else if (startOfMention.start === -1 && this.startPos >= 0) {
                if (caretPosition <= this.startPos) {
                    this.mentionsList.show = false;
                    this.startPos = -1;
                }
                else if (keyCode !== Key.Shift && !event.shiftKey && !event.metaKey && !event.altKey && !event.ctrlKey &&
                    caretPosition > this.startPos) {
                    this.handleKeyDown(event, caretPosition, characterPressed);
                }
            }
            else {
                this.onSelect({ target: this.textAreaInputElement.nativeElement });
            }
        };
    NgMentionsComponent.prototype.onBlur =
        function (event) {
            if (event instanceof FocusEvent && event.relatedTarget) {
                var element = (event.relatedTarget);
                if (element.classList.contains('dropdown-item')) {
                    return;
                }
            }
            this.stopEvent(event);
            if (this.mentionsList) {
                this.mentionsList.show = false;
            }
        };
    NgMentionsComponent.prototype.isPartMention =
        function (part) {
            return typeof part.contents !== 'undefined';
        };
    NgMentionsComponent.prototype.formatMention =
        function (mention) {
            return this._formatMention(mention.contents);
        };
    NgMentionsComponent.prototype.displayTransform =
        function () {
            var _ = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                _[_i] = arguments[_i];
            }
            var replaceIndex = this.markupSearch.groups[this.displayName];
            return _[replaceIndex];
        };
    NgMentionsComponent.prototype._formatMention =
        function (contents) {
            var replaceValue = "$" + this.displayName, replaceIndex;
            var result = contents.replace(this.markupSearch.regEx, replaceValue);
            if (result === replaceValue) {
                replaceIndex = "$" + this.markupSearch.groups[this.displayName];
                result = contents.replace(this.markupSearch.regEx, replaceIndex);
            }
            return result;
        };
    NgMentionsComponent.prototype.stopEvent =
        function (event) {
            if (event.preventDefault) {
                event.preventDefault();
                event.stopPropagation();
                event.stopImmediatePropagation();
            }
        };
    NgMentionsComponent.prototype.setupMentionsList =
        function (caretPosition) {
            this.startPos = caretPosition;
            this.startNode = window.getSelection().anchorNode;
            this.searchString = '';
            this.stopSearch = false;
            this.showMentionsList();
            this.updateMentionsList();
        };
    NgMentionsComponent.prototype.handleKeyDown =
        function (event, caretPosition, characterPressed) {
            var _this = this;
            var keyCode = event.which || event.keyCode;
            if (keyCode === Key.Space) {
                this.startPos = -1;
            }
            else if (keyCode === Key.Backspace && caretPosition > 0) {
                caretPosition--;
                if (caretPosition === this.startPos) {
                    this.stopSearch = true;
                }
                this.mentionsList.show = !this.stopSearch;
            }
            else if (this.mentionsList.show) {
                if (keyCode === Key.Tab || keyCode === Key.Enter) {
                    this.stopEvent(event);
                    this.mentionsList.show = false;
                    var value = this._value;
                    var start = mapPlainTextIndex(value, this.markupSearch, this.startPos, false, this.displayTransform.bind(this));
                    var item = event.item || this.mentionsList.selectedItem;
                    var newValue = replacePlaceholders(item, this.markupSearch);
                    var newDisplayValue = this._formatMention(newValue);
                    caretPosition = this.startPos + newDisplayValue.length;
                    var searchString = this.searchString || '';
                    value = value.substring(0, start) + newValue + value.substring(start + searchString.length + 1, value.length);
                    this.parseLines(value);
                    this.startPos = -1;
                    this.searchString = '';
                    this.stopSearch = true;
                    this.mentionsList.show = false;
                    this.changeDetector.detectChanges();
                    setTimeout(function () {
                        setCaretPosition(_this.textAreaInputElement.nativeElement, caretPosition);
                        _this.onSelect({ target: _this.textAreaInputElement.nativeElement });
                    });
                    return;
                }
                else if (keyCode === Key.Escape) {
                    this.stopEvent(event);
                    this.mentionsList.show = false;
                    this.stopSearch = true;
                    return;
                }
                else if (keyCode === Key.ArrowDown) {
                    this.stopEvent(event);
                    this.mentionsList.selectNextItem();
                    return;
                }
                else if (keyCode === Key.ArrowUp) {
                    this.stopEvent(event);
                    this.mentionsList.selectPreviousItem();
                    return;
                }
                else if (keyCode === Key.Home) {
                    this.stopEvent(event);
                    this.mentionsList.selectFirstItem();
                    return;
                }
                else if (keyCode === Key.End) {
                    this.stopEvent(event);
                    this.mentionsList.selectLastItem();
                    return;
                }
            }
            if (keyCode === Key.ArrowLeft || keyCode === Key.ArrowRight || keyCode === Key.Home || keyCode === Key.End) {
                this.onSelect(event);
                return;
            }
            var mention = this.displayContent.substring(this.startPos + 1, caretPosition);
            if (keyCode !== Key.Backspace) {
                mention += characterPressed;
            }
            this.searchString = mention || '';
            this.updateMentionsList();
        };
    NgMentionsComponent.prototype.getDisplayValue =
        function (item) {
            if (typeof item === 'string') {
                return item;
            }
            else if (item[this.displayName] !== undefined) {
                return item[this.displayName];
            }
            return null;
        };
    NgMentionsComponent.prototype.showMentionsList =
        function () {
            var _this = this;
            if (!this.mentionsList) {
                var componentFactory = this.componentResolver.resolveComponentFactory(NgMentionsListComponent);
                var componentRef = this.viewContainer.createComponent(componentFactory);
                this.mentionsList = componentRef.instance;
                this.mentionsList.itemTemplate = this.mentionListTemplate;
                this.mentionsList.displayTransform = this.displayTransform.bind(this);
                this.mentionsList.itemSelected.subscribe(function (item) {
                    _this.textAreaInputElement.nativeElement.focus();
                    var fakeEvent = { keyCode: Key.Enter, wasSelection: true, item: item };
                    _this.onKeyDown(fakeEvent);
                });
                this.mentionsList.displayTransform = this.getDisplayValue.bind(this);
            }
            this.mentionsList.textAreaElement = this.textAreaInputElement.nativeElement;
            this.mentionsList.show = true;
            this.mentionsList.dropUp = this.dropUp;
            this.mentionsList.activeIndex = 0;
            this.mentionsList.position();
            this.ngZone.run(function () { return _this.mentionsList.resetScroll(); });
        };
    NgMentionsComponent.prototype.updateMentionsList =
        function () {
            var _this = this;
            if (!this.disableSearch) {
                var items = Array.from(this.mentions);
                if (this.searchString) {
                    var searchString = this.searchString.toLowerCase(), searchRegEx_1 = new RegExp(escapeRegExp(searchString), 'i');
                    items = items.filter(function (item) {
                        var value = _this.getDisplayValue(item);
                        return value !== null && searchRegEx_1.test(value);
                    });
                    if (this.maxItems > 0) {
                        items = items.slice(0, this.maxItems);
                    }
                }
                this.mentionsList.items = items;
            }
            else {
                this.search.emit(this.searchString);
            }
        };
    NgMentionsComponent.prototype.parseMarkup =
        function () {
            if (this.mentionMarkup.length === 0 || this.mentionMarkup[0] !== this.triggerChar) {
                throw new Error("ng-mentions markup pattern must start with the trigger character \"" + this.triggerChar + "\"");
            }
            this.markupSearch = markupToRegExp(this.mentionMarkup);
        };
    NgMentionsComponent.prototype.parseLines =
        function (value) {
            var _this = this;
            if (value === void 0) {
                value = '';
            }
            if (value !== this._value) {
                value = value || '';
                var lines = value.split(this.newLine).map(function (line) { return _this.formatMentions(line); });
                var displayContent = lines.map(function (line) { return line.content; }).join('\n');
                if (this.displayContent !== displayContent) {
                    this.lines = lines;
                    this.displayContent = displayContent;
                    this.triggerChange(value);
                }
            }
        };
    NgMentionsComponent.prototype.formatMentions =
        function (line) {
            var _this = this;
            var lineObj = ({ originalContent: line, content: line, parts: [] });
            if (line.length === 0) {
                return lineObj;
            }
            var tags = [], mention;
            var regEx = this.markupSearch.regEx;
            regEx.lastIndex = 0;
            while ((mention = regEx.exec(line)) !== null) {
                tags.push({ indices: { start: mention.index, end: mention.index + mention[0].length } });
            }
            var prevTags = [];
            var content = '';
            tags.slice().sort(function (tagA, tagB) { return tagA.indices.start - tagB.indices.start; }).forEach(function (tag) {
                var expectedLength = tag.indices.end - tag.indices.start;
                var contents = line.slice(tag.indices.start, tag.indices.end);
                if (contents.length === expectedLength) {
                    var prevIndex = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
                    var before = line.slice(prevIndex, tag.indices.start);
                    var partMention = ({ contents: contents, tag: tag });
                    lineObj.parts.push(before);
                    lineObj.parts.push(partMention);
                    prevTags.push(tag);
                    content += before + _this.formatMention(partMention);
                }
            });
            var remainingStart = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
            var remaining = line.slice(remainingStart);
            lineObj.parts.push(remaining);
            content += remaining;
            lineObj.content = content;
            return lineObj;
        };
    NgMentionsComponent.prototype.refreshStyles =
        function () {
            var _this = this;
            var element = this.textAreaInputElement.nativeElement;
            var computedStyle = getComputedStyle(element);
            this.highlighterStyle = {};
            styleProperties.forEach(function (prop) {
                _this.highlighterStyle[prop] = computedStyle[prop];
            });
            this.changeDetector.detectChanges();
        };
    NgMentionsComponent.prototype.triggerChange =
        function (value) {
            this._value = value;
            this.valueChanges.emit(this._value);
            this.changeDetector.detectChanges();
        };
    NgMentionsComponent.decorators = [
        { type: core.Component, args: [{
                    moduleId: module.id,
                    exportAs: 'ngMentions',
                    selector: 'ng-mentions',
                    template: "\n      <div #highlighter class=\"highlighter\" [ngClass]=\"textAreaClassNames\" [attr.readonly]=\"readonly\"\n           [ngStyle]=\"highlighterStyle\">\n          <div *ngFor=\"let line of lines\">\n              <ng-container *ngFor=\"let part of line.parts\">\n                  <highlighted *ngIf=\"isPartMention(part)\" [tag]=\"part.tag\">{{formatMention(part)}}</highlighted>\n                  <ng-container *ngIf=\"!isPartMention(part)\">{{part}}</ng-container>\n              </ng-container>\n              &nbsp;\n          </div>\n      </div>\n      <textarea\n        #input\n        [rows]=\"rows\"\n        [cols]=\"columns\"\n        [ngModel]=\"displayContent\"\n        [ngClass]=\"textAreaClassNames\"\n        (keydown)=\"onKeyDown($event)\"\n        (blur)=\"onBlur($event)\"\n        (select)=\"onSelect($event)\"\n        (mouseup)=\"onSelect($event)\"\n        (ngModelChange)=\"onChange($event)\"\n        (scroll)=\"onTextAreaScroll()\"\n        [disabled]=\"disabled\"\n        [required]=\"required\"\n        [placeholder]=\"placeholder\"\n      ></textarea>\n  ",
                    styles: [
                        'ng-mentions {position: relative;}',
                        'ng-mentions textarea {position:relative; background-color: transparent !important;}', "ng-mentions .highlighter {\n        position: absolute;\n        top:      0;\n        left:     0;\n        right:    0;\n        bottom:   0;\n        color:    transparent;\n        overflow: hidden !important;\n    }",
                        "ng-mentions highlighted {\n        display:          inline;\n        border-radius:    3px;\n        padding:          1px;\n        margin:           -1px;\n        overflow-wrap:    break-word;\n        background-color: lightblue;\n    }"
                    ],
                    preserveWhitespaces: false,
                    encapsulation: core.ViewEncapsulation.None
                },] },
    ];
    NgMentionsComponent.ctorParameters = function () {
        return [
            { type: core.ElementRef, },
            { type: core.ComponentFactoryResolver, },
            { type: core.ViewContainerRef, },
            { type: core.ChangeDetectorRef, },
            { type: core.NgZone, },
        ];
    };
    NgMentionsComponent.propDecorators = {
        "triggerChar": [{ type: core.Input, args: ['triggerChar',] },],
        "mentionMarkup": [{ type: core.Input, args: ['markup',] },],
        "disableSearch": [{ type: core.Input, args: ['disableSearch',] },],
        "maxItems": [{ type: core.Input, args: ['maxItems',] },],
        "dropUp": [{ type: core.Input, args: ['dropUp',] },],
        "displayName": [{ type: core.Input, args: ['displayName',] },],
        "formClass": [{ type: core.Input, args: ['formClass',] },],
        "placeholder": [{ type: core.Input, args: ['placeholder',] },],
        "value": [{ type: core.Input, args: ['value',] },],
        "required": [{ type: core.Input, args: ['required',] },],
        "disabled": [{ type: core.Input, args: ['disabled',] },],
        "rows": [{ type: core.Input, args: ['rows',] },],
        "columns": [{ type: core.Input, args: ['cols',] },],
        "mentionItems": [{ type: core.Input, args: ['mentions',] },],
        "search": [{ type: core.Output, args: ['search',] },],
        "valueChanges": [{ type: core.Output, args: ['valueChanges',] },],
        "stateChanges": [{ type: core.Output, args: ['stateChanges',] },],
        "mentionListTemplate": [{ type: core.ContentChild, args: [core.TemplateRef,] },],
        "textAreaInputElement": [{ type: core.ViewChild, args: ['input',] },],
        "highlighterElement": [{ type: core.ViewChild, args: ['highlighter',] },],
        "onWindowResize": [{ type: core.HostListener, args: ['window:resize',] },],
    };
    return NgMentionsComponent;
}());
var NgMentionsAccessorDirective = (function () {
    function NgMentionsAccessorDirective(element, host) {
        this.element = element;
        this.host = host;
        this._destroyed = new rxjs.Subject();
    }
    NgMentionsAccessorDirective.prototype.ngOnInit =
        function () {
            var _this = this;
            this.host.valueChanges.pipe(operators.takeUntil(this._destroyed)).subscribe(function (value) { return _this.onChange(value); });
        };
    NgMentionsAccessorDirective.prototype.ngOnDestroy =
        function () {
            this._destroyed.next();
            this._destroyed.complete();
        };
    NgMentionsAccessorDirective.prototype.registerOnChange =
        function (fn) {
            this._onChange = fn;
        };
    NgMentionsAccessorDirective.prototype.registerOnTouched =
        function (fn) {
            this._onTouch = fn;
        };
    NgMentionsAccessorDirective.prototype.setDisabledState =
        function (isDisabled) {
            this.host.disabled = isDisabled;
        };
    NgMentionsAccessorDirective.prototype.writeValue =
        function (value) {
            if (typeof value === 'string' || value === null) {
                this.host.value = value;
            }
        };
    NgMentionsAccessorDirective.prototype.onChange =
        function (value) {
            if (this._onChange) {
                this._onChange(value);
            }
        };
    NgMentionsAccessorDirective.prototype.onTouched =
        function () {
            if (this._onTouch) {
                this._onTouch();
            }
        };
    NgMentionsAccessorDirective.decorators = [
        { type: core.Directive, args: [{
                    exportAs: 'ngMentions',
                    selector: 'ng-mentions',
                    host: { '(change)': 'onChange($event)', '(touch)': 'onTouched()' },
                    providers: [{ provide: forms.NG_VALUE_ACCESSOR, useExisting: core.forwardRef(function () { return NgMentionsAccessorDirective; }), multi: true }]
                },] },
    ];
    NgMentionsAccessorDirective.ctorParameters = function () {
        return [
            { type: core.ElementRef, },
            { type: NgMentionsComponent, },
        ];
    };
    return NgMentionsAccessorDirective;
}());
var HighlightedDirective = (function () {
    function HighlightedDirective() {
    }
    HighlightedDirective.decorators = [
        { type: core.Directive, args: [{ selector: 'highlighted' },] },
    ];
    HighlightedDirective.ctorParameters = function () { return []; };
    HighlightedDirective.propDecorators = {
        "tag": [{ type: core.Input },],
    };
    return HighlightedDirective;
}());
var EXPORT_DIRECTIVES = [NgMentionsComponent, NgMentionsAccessorDirective];
var DECLARATIONS = [NgMentionsComponent, NgMentionsAccessorDirective, NgMentionsListComponent, HighlightedDirective];
var NgMentionsModule = (function () {
    function NgMentionsModule() {
    }
    NgMentionsModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [common.CommonModule, forms.FormsModule],
                    exports: EXPORT_DIRECTIVES,
                    declarations: DECLARATIONS,
                    entryComponents: [NgMentionsListComponent]
                },] },
    ];
    NgMentionsModule.ctorParameters = function () { return []; };
    return NgMentionsModule;
}());

exports.NgMentionsAccessorDirective = NgMentionsAccessorDirective;
exports.NgMentionsComponent = NgMentionsComponent;
exports.NgMentionsModule = NgMentionsModule;
exports.ɵb = HighlightedDirective;
exports.ɵa = NgMentionsListComponent;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ng-mentions.umd.js.map
