/**
 * The MIT License
 *
 * Copyright (c) 2010-2016 Google, Inc. http://angularjs.org
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

import { Component, ElementRef, EventEmitter, HostBinding, TemplateRef, ViewChild, ViewEncapsulation, ChangeDetectorRef, ComponentFactoryResolver, ContentChild, HostListener, Input, NgZone, Output, ViewContainerRef, Directive, forwardRef, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { NG_VALUE_ACCESSOR, FormsModule } from '@angular/forms';
import { takeUntil } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/** @enum {number} */
var Key = {
    Backspace: 8,
    Tab: 9,
    Enter: 13,
    Shift: 16,
    Escape: 27,
    Space: 32,
    End: 35,
    Home: 36,
    ArrowLeft: 37,
    ArrowUp: 38,
    ArrowRight: 39,
    ArrowDown: 40,
};
Key[Key.Backspace] = "Backspace";
Key[Key.Tab] = "Tab";
Key[Key.Enter] = "Enter";
Key[Key.Shift] = "Shift";
Key[Key.Escape] = "Escape";
Key[Key.Space] = "Space";
Key[Key.End] = "End";
Key[Key.Home] = "Home";
Key[Key.ArrowLeft] = "ArrowLeft";
Key[Key.ArrowUp] = "ArrowUp";
Key[Key.ArrowRight] = "ArrowRight";
Key[Key.ArrowDown] = "ArrowDown";

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

/**
 * @record
 */

/**
 * @record
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var styleProperties = Object.freeze([
    'direction',
    'boxSizing',
    'width',
    'height', 'overflowX',
    'overflowY',
    'borderTopWidth', 'borderRightWidth', 'borderBottomWidth', 'borderLeftWidth', 'borderStyle',
    'paddingTop', 'paddingRight', 'paddingBottom', 'paddingLeft',
    'fontStyle', 'fontVariant', 'fontWeight', 'fontStretch', 'fontSize', 'fontSizeAdjust', 'lineHeight', 'fontFamily',
    'textAlign', 'textTransform', 'textIndent',
    'textDecoration',
    'letterSpacing', 'wordSpacing',
    'tabSize', 'MozTabSize'
]);
var isBrowser = typeof window !== 'undefined';
var isFirefox = isBrowser && (/** @type {?} */ (window)).mozInnerScreenX != null;
/**
 * @param {?=} element
 * @return {?}
 */
function isInputOrTextAreaElement(element) {
    return element !== null &&
        (element.nodeName === 'INPUT' || element.nodeName === 'TEXTAREA' || element.nodeName === '#text');
}
/**
 * @param {?} value
 * @param {?} start
 * @param {?} end
 * @param {?} insert
 * @return {?}
 */
function spliceString(value, start, end, insert) {
    return value.substring(0, start) + insert + value.substring(end) + '';
}
/**
 * @param {?} value
 * @param {?} mentionMarkup
 * @param {?} textIterator
 * @param {?} markupIterator
 * @param {?} displayTransform
 * @return {?}
 */
function iterateMentionsMarkup(value, mentionMarkup, textIterator, markupIterator, displayTransform) {
    var /** @type {?} */ match, /** @type {?} */ start = 0, /** @type {?} */ currentPlainTextIndex = 0;
    var /** @type {?} */ regEx = mentionMarkup.regEx;
    regEx.lastIndex = 0;
    while ((match = regEx.exec(value)) !== null) {
        var /** @type {?} */ display = displayTransform.apply(void 0, match);
        var /** @type {?} */ substr = value.substring(start, match.index);
        textIterator(substr, start, currentPlainTextIndex);
        currentPlainTextIndex += substr.length;
        markupIterator(match[0], match.index, currentPlainTextIndex, display);
        currentPlainTextIndex += display.length;
        start = regEx.lastIndex;
    }
    if (start < value.length) {
        textIterator(value.substring(start), start, currentPlainTextIndex);
    }
}
/**
 * @param {?} value
 * @param {?} mentionMarkup
 * @param {?} markupIterator
 * @param {?} displayTransform
 * @return {?}
 */
function iterateOnlyMentionsMarkup(value, mentionMarkup, markupIterator, displayTransform) {
    var /** @type {?} */ match, /** @type {?} */ start = 0, /** @type {?} */ currentPlainTextIndex = 0;
    var /** @type {?} */ regEx = mentionMarkup.regEx;
    regEx.lastIndex = 0;
    while ((match = regEx.exec(value)) !== null) {
        var /** @type {?} */ display = displayTransform.apply(void 0, match);
        var /** @type {?} */ substr = value.substring(start, match.index);
        currentPlainTextIndex += substr.length;
        markupIterator(match[0], match.index, currentPlainTextIndex, display);
        currentPlainTextIndex += display.length;
        start = regEx.lastIndex;
    }
}
/**
 * @param {?} value
 * @param {?} mentionMarkup
 * @param {?} indexInPlainText
 * @param {?} toEndOfMarkup
 * @param {?} displayTransform
 * @return {?}
 */
function mapPlainTextIndex(value, mentionMarkup, indexInPlainText, toEndOfMarkup, displayTransform) {
    if (isNaN(indexInPlainText)) {
        return indexInPlainText;
    }
    var /** @type {?} */ result;
    var /** @type {?} */ textIterator = function (matchString, index, substringPlainTextIndex) {
        if (typeof result !== 'undefined') {
            return;
        }
        if (substringPlainTextIndex + matchString.length >= indexInPlainText) {
            result = index + indexInPlainText - substringPlainTextIndex;
        }
    };
    var /** @type {?} */ markupIterator = function (matchString, index, mentionPlainTextIndex, display) {
        if (typeof result !== 'undefined') {
            return;
        }
        if (mentionPlainTextIndex + display.length > indexInPlainText) {
            result = index + (toEndOfMarkup ? matchString.length : 0);
        }
    };
    iterateMentionsMarkup(value, mentionMarkup, textIterator, markupIterator, displayTransform);
    return typeof result !== 'undefined' ? result : value.length;
}
/**
 * @param {?} element
 * @return {?}
 */
function getCaretPosition(element) {
    if (isInputOrTextAreaElement(element)) {
        var /** @type {?} */ value = element.value;
        return value.slice(0, element.selectionStart).length;
    }
    var /** @type {?} */ selection = window.getSelection();
    if (selection.rangeCount > 0) {
        var /** @type {?} */ range = selection.getRangeAt(0);
        return range.startOffset;
    }
    return 0;
}
/**
 * @param {?} element
 * @param {?} position
 * @return {?}
 */
function getCaretCoordinates(element, position) {
    var /** @type {?} */ coords = { top: 0, left: 0 };
    if (!isBrowser) {
        return coords;
    }
    var /** @type {?} */ div = document.createElement('div');
    document.body.appendChild(div);
    var /** @type {?} */ style = div.style;
    var /** @type {?} */ computed = window.getComputedStyle ? getComputedStyle(element) : (/** @type {?} */ (element)).currentStyle;
    style.whiteSpace = 'pre-wrap';
    if (element.nodeName !== 'INPUT') {
        style.wordWrap = 'break-word';
    }
    style.position = 'absolute';
    style.visibility = 'hidden';
    styleProperties.forEach(function (prop) { return style[prop] = computed[prop]; });
    if (isFirefox) {
        if (element.scrollHeight > parseInt(computed.height, 10)) {
            style.overflowY = 'scroll';
        }
    }
    else {
        style.overflow = 'hidden';
    }
    div.textContent = element.value.substring(0, position);
    if (element.nodeName === 'INPUT') {
        div.textContent = div.textContent.replace(/\s/g, '\u00a0');
    }
    var /** @type {?} */ span = document.createElement('span');
    span.textContent = element.value.substring(position) || '.';
    div.appendChild(span);
    coords = {
        top: span.offsetTop + parseInt(computed['borderTopWidth'], 10),
        left: span.offsetLeft + parseInt(computed['borderLeftWidth'], 10)
    };
    document.body.removeChild(div);
    return coords;
}
/**
 * @param {?} element
 * @param {?} position
 * @return {?}
 */
function setCaretPosition(element, position) {
    if (isInputOrTextAreaElement(element) && element.selectionStart !== undefined) {
        element.focus();
        element.setSelectionRange(position, position);
    }
    else {
        var /** @type {?} */ range = document.createRange();
        range.setStart(element, position);
        range.collapse(true);
        var /** @type {?} */ selection = window.getSelection();
        selection.removeAllRanges();
        selection.addRange(range);
    }
}
/**
 * @param {?} str
 * @return {?}
 */
function escapeRegExp(str) {
    return str.replace(/[-\/\\^$*+?.()|[\]{}]/g, '\\$&');
}
/**
 * @record
 */

/**
 * @param {?} markup
 * @return {?}
 */
function markupToRegExp(markup) {
    var /** @type {?} */ markupPattern = escapeRegExp(markup);
    var /** @type {?} */ placeholderRegExp = /__([\w]+)__/g;
    var /** @type {?} */ placeholders = {};
    var /** @type {?} */ groups = /[\[\]()]+/.test(markup);
    var /** @type {?} */ match, /** @type {?} */ i = groups ? 1 : 0, /** @type {?} */ placeholderExclusion = groups ? '^\\)\\]' : '\\S';
    do {
        match = placeholderRegExp.exec(markupPattern);
        if (match) {
            var /** @type {?} */ placeholder = match[1];
            markupPattern = markupPattern.replace("__" + placeholder + "__", "([" + placeholderExclusion + "]+)");
            placeholders[placeholder] = ++i;
        }
    } while (match);
    return { markup: markup, regEx: new RegExp('(' + markupPattern + ')', 'ig'), groups: placeholders };
}
/**
 * @param {?} value
 * @param {?} mentionMarkup
 * @param {?} displayTransform
 * @return {?}
 */
function getPlainText(value, mentionMarkup, displayTransform) {
    mentionMarkup.regEx.lastIndex = 0;
    return value.replace(mentionMarkup.regEx, displayTransform);
}
/**
 * @param {?} item
 * @param {?} markupMention
 * @return {?}
 */
function replacePlaceholders(item, markupMention) {
    var /** @type {?} */ result = markupMention.markup + '';
    Object.keys(markupMention.groups)
        .forEach(function (key) { return result = result.replace(new RegExp("__" + key + "__", 'g'), typeof item === 'string' ? item : item[key]); });
    return result;
}
/**
 * @param {?} value
 * @param {?} markupMention
 * @param {?} plainTextValue
 * @param {?=} selectionStartBeforeChange
 * @param {?=} selectionEndBeforeChange
 * @param {?=} selectionEndAfterChange
 * @param {?=} displayTransform
 * @return {?}
 */
function applyChangeToValue(value, markupMention, plainTextValue, selectionStartBeforeChange, selectionEndBeforeChange, selectionEndAfterChange, displayTransform) {
    if (selectionStartBeforeChange === void 0) { selectionStartBeforeChange = 0; }
    if (selectionEndBeforeChange === void 0) { selectionEndBeforeChange = 0; }
    if (selectionEndAfterChange === void 0) { selectionEndAfterChange = 0; }
    var /** @type {?} */ oldPlainTextValue = getPlainText(value, markupMention, displayTransform);
    var /** @type {?} */ lengthDelta = oldPlainTextValue.length - plainTextValue.length;
    if (!selectionStartBeforeChange) {
        selectionStartBeforeChange = selectionEndBeforeChange + lengthDelta;
    }
    if (!selectionEndBeforeChange) {
        selectionEndBeforeChange = selectionStartBeforeChange;
    }
    if (selectionStartBeforeChange === selectionEndBeforeChange && selectionEndBeforeChange === selectionEndAfterChange &&
        oldPlainTextValue.length === plainTextValue.length) {
        selectionStartBeforeChange--;
    }
    var /** @type {?} */ insert = plainTextValue.slice(selectionStartBeforeChange, selectionEndAfterChange);
    var /** @type {?} */ spliceStart = Math.min(selectionStartBeforeChange, selectionEndAfterChange);
    var /** @type {?} */ spliceEnd = selectionEndBeforeChange;
    if (selectionStartBeforeChange === selectionEndAfterChange) {
        spliceEnd = Math.max(selectionEndBeforeChange, selectionStartBeforeChange + lengthDelta);
    }
    return spliceString(value, mapPlainTextIndex(value, markupMention, spliceStart, false, displayTransform), mapPlainTextIndex(value, markupMention, spliceEnd, true, displayTransform), insert);
}
/**
 * @param {?} value
 * @param {?} mentionMarkup
 * @param {?} indexInPlainText
 * @param {?} displayTransform
 * @return {?}
 */
function findStartOfMentionInPlainText(value, mentionMarkup, indexInPlainText, displayTransform) {
    var /** @type {?} */ result = { start: -1, end: -1 };
    var /** @type {?} */ markupIterator = function (matchString, index, mentionPlainTextIndex, display) {
        if (mentionPlainTextIndex < indexInPlainText && mentionPlainTextIndex + display.length > indexInPlainText) {
            result = { start: mentionPlainTextIndex, end: mentionPlainTextIndex + display.length };
            return true;
        }
        return false;
    };
    iterateOnlyMentionsMarkup(value, mentionMarkup, markupIterator, displayTransform);
    return result;
}
/**
 * @param {?} value
 * @param {?} mentionMarkup
 * @param {?} indexInPlainText
 * @param {?} displayTransform
 * @return {?}
 */
function getBoundsOfMentionAtPosition(value, mentionMarkup, indexInPlainText, displayTransform) {
    return findStartOfMentionInPlainText(value, mentionMarkup, indexInPlainText, displayTransform);
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var NgMentionsListComponent = /** @class */ (function () {
    function NgMentionsListComponent() {
        this.activeIndex = -1;
        this.itemSelected = new EventEmitter();
        this.show = false;
        this.dropUp = false;
        this._top = 0;
        this._left = 0;
    }
    Object.defineProperty(NgMentionsListComponent.prototype, "selectedItem", {
        get: /**
         * @return {?}
         */
        function () {
            return this.activeIndex >= 0 && this.items[this.activeIndex] !== undefined ? this.items[this.activeIndex] : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "top", {
        get: /**
         * @return {?}
         */
        function () {
            return this._top + this.adjustTop + 'px';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "left", {
        get: /**
         * @return {?}
         */
        function () {
            return this._left + 'px';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsListComponent.prototype, "noItems", {
        get: /**
         * @return {?}
         */
        function () {
            return !Array.isArray(this.items) || this.items.length === 0;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NgMentionsListComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        if (!this.itemTemplate) {
            this.itemTemplate = this.defaultItemTemplate;
        }
    };
    /**
     * @param {?} event
     * @param {?} activeIndex
     * @param {?} item
     * @return {?}
     */
    NgMentionsListComponent.prototype.onItemClick = /**
     * @param {?} event
     * @param {?} activeIndex
     * @param {?} item
     * @return {?}
     */
    function (event, activeIndex, item) {
        event.preventDefault();
        this.activeIndex = activeIndex;
        this.itemSelected.emit(item);
    };
    /**
     * @return {?}
     */
    NgMentionsListComponent.prototype.selectFirstItem = /**
     * @return {?}
     */
    function () {
        this.activeIndex = 0;
        this.resetScroll();
    };
    /**
     * @return {?}
     */
    NgMentionsListComponent.prototype.selectPreviousItem = /**
     * @return {?}
     */
    function () {
        if (this.activeIndex > 0) {
            this.activeIndex--;
        }
        this.scrollToActiveItem();
    };
    /**
     * @return {?}
     */
    NgMentionsListComponent.prototype.selectNextItem = /**
     * @return {?}
     */
    function () {
        if (this.activeIndex < this.items.length - 1) {
            this.activeIndex++;
            this.scrollToActiveItem();
        }
    };
    /**
     * @return {?}
     */
    NgMentionsListComponent.prototype.selectLastItem = /**
     * @return {?}
     */
    function () {
        this.activeIndex = this.items.length > 0 ? this.items.length - 1 : 0;
        this.scrollToActiveItem();
    };
    /**
     * @return {?}
     */
    NgMentionsListComponent.prototype.position = /**
     * @return {?}
     */
    function () {
        var /** @type {?} */ element = this.textAreaElement;
        var /** @type {?} */ coords = getCaretCoordinates(element, element.selectionStart);
        this._top = coords.top;
        this._left = coords.left + element.offsetLeft;
        this.list.nativeElement.scrollTop = 0;
    };
    /**
     * @return {?}
     */
    NgMentionsListComponent.prototype.resetScroll = /**
     * @return {?}
     */
    function () {
        this.list.nativeElement.scrollTop = 0;
    };
    /**
     * @param {?} item
     * @return {?}
     */
    NgMentionsListComponent.prototype.transformItem = /**
     * @param {?} item
     * @return {?}
     */
    function (item) {
        return this.displayTransform(item) || item;
    };
    Object.defineProperty(NgMentionsListComponent.prototype, "adjustTop", {
        get: /**
         * @return {?}
         */
        function () {
            var /** @type {?} */ adjust = 0;
            if (!this.dropUp) {
                var /** @type {?} */ computed = getComputedStyle(this.textAreaElement);
                adjust = parseInt(computed.fontSize, 10) + this.textAreaElement.offsetTop;
            }
            return adjust;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NgMentionsListComponent.prototype.scrollToActiveItem = /**
     * @return {?}
     */
    function () {
        var /** @type {?} */ element = /** @type {?} */ (this.list.nativeElement);
        if (this.activeIndex === 0) {
            element.scrollTop = 0;
        }
        else {
            var /** @type {?} */ activeElement = /** @type {?} */ (element.querySelector('li.active'));
            if (activeElement) {
                element.scrollTop = activeElement.offsetTop;
            }
        }
    };
    NgMentionsListComponent.decorators = [
        { type: Component, args: [{
                    selector: 'mentions-list',
                    template: "\n      <ng-template #defaultItemTemplate let-item=\"item\">\n          {{transformItem(item)}}\n      </ng-template>\n      <ul #list class=\"dropdown-menu scrollable-menu\">\n          <li *ngFor=\"let item of items; let i = index\" [class.active]=\"activeIndex === i\">\n              <a href class=\"dropdown-item\" (click)=\"onItemClick($event, i, item)\">\n                  <ng-template [ngTemplateOutlet]=\"itemTemplate\"\n                               [ngTemplateOutletContext]=\"{item:item,index:i}\"></ng-template>\n              </a>\n          </li>\n      </ul>\n  ",
                    styles: [
                        'mentions-list {position: absolute;display: none;}', 'mentions-list.drop-up {transform: translateY(-100%);}',
                        'mentions-list.show {display: block;} mentions-list.no-items {display: none;}',
                        'mentions-list .scrollable-menu {display: block;height: auto;max-height:300px;overflow:auto;}',
                        'mentions-list li.active {background: #f7f7f9;}'
                    ],
                    encapsulation: ViewEncapsulation.None
                },] },
    ];
    /** @nocollapse */
    NgMentionsListComponent.ctorParameters = function () { return []; };
    NgMentionsListComponent.propDecorators = {
        "defaultItemTemplate": [{ type: ViewChild, args: ['defaultItemTemplate',] },],
        "list": [{ type: ViewChild, args: ['list',] },],
        "show": [{ type: HostBinding, args: ['class.show',] },],
        "dropUp": [{ type: HostBinding, args: ['class.drop-up',] },],
        "top": [{ type: HostBinding, args: ['style.top',] },],
        "left": [{ type: HostBinding, args: ['style.left',] },],
        "noItems": [{ type: HostBinding, args: ['class.no-items',] },],
    };
    return NgMentionsListComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * The Mentions Component
 */
var NgMentionsComponent = /** @class */ (function () {
    function NgMentionsComponent(element, componentResolver, viewContainer, changeDetector, ngZone) {
        this.element = element;
        this.componentResolver = componentResolver;
        this.viewContainer = viewContainer;
        this.changeDetector = changeDetector;
        this.ngZone = ngZone;
        /**
         * The character to trigger the mentions list when a user is typing in the input field
         */
        this.triggerChar = '@';
        /**
         * The markup used to format a mention in the model value
         */
        this.mentionMarkup = '@[__display__](__type__:__id__)';
        /**
         * Optional. When using a custom search (i.e. an API call), the internal searching capability should be disabled.
         */
        this.disableSearch = false;
        /**
         * Only used when internal search is not disabled. This limits the maximum number of items to display in the search
         * result list.
         */
        this.maxItems = -1;
        /**
         * Used to cause the search result list to display in a "drop up" fashion, instead of a typical dropdown.
         */
        this.dropUp = false;
        /**
         * If the supplied mentions are a list of objects, this is the name of the property used to display
         * the mention in the search result list and when formatting a mention in the displayed text.
         */
        this.displayName = 'display';
        /**
         * An event emitted, after the trigger character has been typed, with the user-entered search string.
         */
        this.search = new EventEmitter();
        this.valueChanges = new EventEmitter();
        this.stateChanges = new Subject();
        this.displayContent = '';
        this.lines = [];
        this.highlighterStyle = {};
        this.textAreaClassNames = {};
        this.mentions = [];
        this._value = '';
        this._rows = 1;
        this._columns = 1;
        this.stopSearch = false;
        this._destroyed = new Subject();
        this.newLine = /\n/g;
        this._errorState = false;
    }
    Object.defineProperty(NgMentionsComponent.prototype, "formClass", {
        get: /**
         * Classes for textarea
         * @return {?}
         */
        function () {
            return Object.keys(this.textAreaClassNames).join(' ');
        },
        set: /**
         * @param {?} classNames
         * @return {?}
         */
        function (classNames) {
            var _this = this;
            this.textAreaClassNames = {};
            Array.from(classNames.split(' ')).forEach(function (className) {
                _this.textAreaClassNames[className] = true;
            });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () {
            return this._value;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this.parseLines(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "required", {
        get: /**
         * @return {?}
         */
        function () {
            return this._required;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this._required = value;
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "disabled", {
        get: /**
         * @return {?}
         */
        function () {
            return this._disabled;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this._disabled = value;
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "rows", {
        get: /**
         * @return {?}
         */
        function () {
            return this._rows;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this._rows = Math.max(1, /** @type {?} */ (value));
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "columns", {
        get: /**
         * @return {?}
         */
        function () {
            return this._columns;
        },
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this._columns = Math.max(1, /** @type {?} */ (value));
            this.refreshStyles();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "mentionItems", {
        set: /**
         * The list of mentions to display, or filter, in the search result list.
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this.mentions = value;
            if (this.disableSearch && this.mentionsList) {
                this.mentionsList.items = value;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "readonly", {
        get: /**
         * @return {?}
         */
        function () {
            return this.disabled ? 'readonly' : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgMentionsComponent.prototype, "errorState", {
        get: /**
         * @return {?}
         */
        function () {
            return this._errorState;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        this.parseMarkup();
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    NgMentionsComponent.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if ('markup' in changes) {
            this.parseMarkup();
        }
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        this.parseLines(this._value);
        this.refreshStyles();
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.ngAfterViewChecked = /**
     * @return {?}
     */
    function () {
        this.refreshStyles();
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this._destroyed.next();
        this._destroyed.complete();
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.onWindowResize = /**
     * @return {?}
     */
    function () {
        this.refreshStyles();
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.onTextAreaScroll = /**
     * @return {?}
     */
    function () {
        this.highlighterElement.nativeElement.scrollTop = this.textAreaInputElement.nativeElement.scrollTop;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgMentionsComponent.prototype.onSelect = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.selectionStart = event.target.selectionStart;
        this.selectionEnd = event.target.selectionEnd;
    };
    /**
     * @param {?} newPlainTextValue
     * @return {?}
     */
    NgMentionsComponent.prototype.onChange = /**
     * @param {?} newPlainTextValue
     * @return {?}
     */
    function (newPlainTextValue) {
        var _this = this;
        var /** @type {?} */ value = this._value;
        var /** @type {?} */ displayTransform = this.displayTransform.bind(this);
        var /** @type {?} */ selectionStart = this.textAreaInputElement.nativeElement.selectionStart;
        var /** @type {?} */ selectionEnd = this.textAreaInputElement.nativeElement.selectionEnd;
        var /** @type {?} */ bounds = getBoundsOfMentionAtPosition(newPlainTextValue, this.markupSearch, selectionStart, displayTransform);
        if (bounds.start !== -1) {
            newPlainTextValue = newPlainTextValue.substring(0, bounds.start) + newPlainTextValue.substring(bounds.end);
        }
        var /** @type {?} */ newValue = applyChangeToValue(value, this.markupSearch, newPlainTextValue, this.selectionStart, this.selectionEnd, selectionEnd, displayTransform);
        var /** @type {?} */ startOfMention = findStartOfMentionInPlainText(value, this.markupSearch, selectionStart, displayTransform);
        if (startOfMention.start > -1 && this.selectionEnd > startOfMention.start) {
            selectionStart = startOfMention.start;
            selectionEnd = selectionStart;
        }
        this.selectionStart = Math.max(selectionStart, 0);
        this.selectionEnd = Math.max(selectionEnd, 0);
        this.parseLines(newValue);
        if (this.selectionEnd > 0) {
            setTimeout(function () { return setCaretPosition(_this.textAreaInputElement.nativeElement, _this.selectionEnd); });
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgMentionsComponent.prototype.onKeyDown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        var /** @type {?} */ caretPosition = getCaretPosition(this.textAreaInputElement.nativeElement);
        var /** @type {?} */ characterPressed = event.key;
        var /** @type {?} */ keyCode = event.which || event.keyCode;
        if (!characterPressed) {
            var /** @type {?} */ characterCode = event.which || event.keyCode;
            characterPressed = String.fromCharCode(characterCode);
            if (!event.shiftKey && (characterCode >= 65 && characterCode <= 90)) {
                characterPressed = String.fromCharCode(characterCode + 32);
            }
        }
        if (keyCode === Key.Enter && event.wasSelection && caretPosition < this.startPos) {
            caretPosition = this.startNode.length;
            setCaretPosition(this.startNode, caretPosition);
        }
        var /** @type {?} */ startOfMention = findStartOfMentionInPlainText(this._value, this.markupSearch, caretPosition, this.displayTransform.bind(this));
        if (characterPressed === this.triggerChar) {
            this.setupMentionsList(caretPosition);
        }
        else if (startOfMention.start === -1 && this.startPos >= 0) {
            if (caretPosition <= this.startPos) {
                this.mentionsList.show = false;
                this.startPos = -1;
            }
            else if (keyCode !== Key.Shift && !event.shiftKey && !event.metaKey && !event.altKey && !event.ctrlKey &&
                caretPosition > this.startPos) {
                this.handleKeyDown(event, caretPosition, characterPressed);
            }
        }
        else {
            this.onSelect({ target: this.textAreaInputElement.nativeElement });
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgMentionsComponent.prototype.onBlur = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (event instanceof FocusEvent && event.relatedTarget) {
            var /** @type {?} */ element = /** @type {?} */ (event.relatedTarget);
            if (element.classList.contains('dropdown-item')) {
                return;
            }
        }
        this.stopEvent(event);
        if (this.mentionsList) {
            this.mentionsList.show = false;
        }
    };
    /**
     * @param {?} part
     * @return {?}
     */
    NgMentionsComponent.prototype.isPartMention = /**
     * @param {?} part
     * @return {?}
     */
    function (part) {
        return typeof part.contents !== 'undefined';
    };
    /**
     * @param {?} mention
     * @return {?}
     */
    NgMentionsComponent.prototype.formatMention = /**
     * @param {?} mention
     * @return {?}
     */
    function (mention) {
        return this._formatMention(mention.contents);
    };
    /**
     * @param {...?} _
     * @return {?}
     */
    NgMentionsComponent.prototype.displayTransform = /**
     * @param {...?} _
     * @return {?}
     */
    function () {
        var _ = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            _[_i] = arguments[_i];
        }
        var /** @type {?} */ replaceIndex = this.markupSearch.groups[this.displayName];
        return _[replaceIndex];
    };
    /**
     * @param {?} contents
     * @return {?}
     */
    NgMentionsComponent.prototype._formatMention = /**
     * @param {?} contents
     * @return {?}
     */
    function (contents) {
        var /** @type {?} */ replaceValue = "$" + this.displayName, /** @type {?} */ replaceIndex;
        var /** @type {?} */ result = contents.replace(this.markupSearch.regEx, replaceValue);
        if (result === replaceValue) {
            replaceIndex = "$" + this.markupSearch.groups[this.displayName];
            result = contents.replace(this.markupSearch.regEx, replaceIndex);
        }
        return result;
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgMentionsComponent.prototype.stopEvent = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (event.preventDefault) {
            event.preventDefault();
            event.stopPropagation();
            event.stopImmediatePropagation();
        }
    };
    /**
     * @param {?} caretPosition
     * @return {?}
     */
    NgMentionsComponent.prototype.setupMentionsList = /**
     * @param {?} caretPosition
     * @return {?}
     */
    function (caretPosition) {
        this.startPos = caretPosition;
        this.startNode = window.getSelection().anchorNode;
        this.searchString = '';
        this.stopSearch = false;
        this.showMentionsList();
        this.updateMentionsList();
    };
    /**
     * @param {?} event
     * @param {?} caretPosition
     * @param {?} characterPressed
     * @return {?}
     */
    NgMentionsComponent.prototype.handleKeyDown = /**
     * @param {?} event
     * @param {?} caretPosition
     * @param {?} characterPressed
     * @return {?}
     */
    function (event, caretPosition, characterPressed) {
        var _this = this;
        var /** @type {?} */ keyCode = event.which || event.keyCode;
        if (keyCode === Key.Space) {
            this.startPos = -1;
        }
        else if (keyCode === Key.Backspace && caretPosition > 0) {
            caretPosition--;
            if (caretPosition === this.startPos) {
                this.stopSearch = true;
            }
            this.mentionsList.show = !this.stopSearch;
        }
        else if (this.mentionsList.show) {
            if (keyCode === Key.Tab || keyCode === Key.Enter) {
                this.stopEvent(event);
                this.mentionsList.show = false;
                var /** @type {?} */ value = this._value;
                var /** @type {?} */ start = mapPlainTextIndex(value, this.markupSearch, this.startPos, false, this.displayTransform.bind(this));
                var /** @type {?} */ item = event.item || this.mentionsList.selectedItem;
                var /** @type {?} */ newValue = replacePlaceholders(item, this.markupSearch);
                var /** @type {?} */ newDisplayValue = this._formatMention(newValue);
                caretPosition = this.startPos + newDisplayValue.length;
                var /** @type {?} */ searchString = this.searchString || '';
                value = value.substring(0, start) + newValue + value.substring(start + searchString.length + 1, value.length);
                this.parseLines(value);
                this.startPos = -1;
                this.searchString = '';
                this.stopSearch = true;
                this.mentionsList.show = false;
                this.changeDetector.detectChanges();
                setTimeout(function () {
                    setCaretPosition(_this.textAreaInputElement.nativeElement, caretPosition);
                    _this.onSelect({ target: _this.textAreaInputElement.nativeElement });
                });
                return;
            }
            else if (keyCode === Key.Escape) {
                this.stopEvent(event);
                this.mentionsList.show = false;
                this.stopSearch = true;
                return;
            }
            else if (keyCode === Key.ArrowDown) {
                this.stopEvent(event);
                this.mentionsList.selectNextItem();
                return;
            }
            else if (keyCode === Key.ArrowUp) {
                this.stopEvent(event);
                this.mentionsList.selectPreviousItem();
                return;
            }
            else if (keyCode === Key.Home) {
                this.stopEvent(event);
                this.mentionsList.selectFirstItem();
                return;
            }
            else if (keyCode === Key.End) {
                this.stopEvent(event);
                this.mentionsList.selectLastItem();
                return;
            }
        }
        if (keyCode === Key.ArrowLeft || keyCode === Key.ArrowRight || keyCode === Key.Home || keyCode === Key.End) {
            this.onSelect(event);
            return;
        }
        var /** @type {?} */ mention = this.displayContent.substring(this.startPos + 1, caretPosition);
        if (keyCode !== Key.Backspace) {
            mention += characterPressed;
        }
        this.searchString = mention || '';
        this.updateMentionsList();
    };
    /**
     * @param {?} item
     * @return {?}
     */
    NgMentionsComponent.prototype.getDisplayValue = /**
     * @param {?} item
     * @return {?}
     */
    function (item) {
        if (typeof item === 'string') {
            return item;
        }
        else if (item[this.displayName] !== undefined) {
            return item[this.displayName];
        }
        return null;
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.showMentionsList = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this.mentionsList) {
            var /** @type {?} */ componentFactory = this.componentResolver.resolveComponentFactory(NgMentionsListComponent);
            var /** @type {?} */ componentRef = this.viewContainer.createComponent(componentFactory);
            this.mentionsList = componentRef.instance;
            this.mentionsList.itemTemplate = this.mentionListTemplate;
            this.mentionsList.displayTransform = this.displayTransform.bind(this);
            this.mentionsList.itemSelected.subscribe(function (item) {
                _this.textAreaInputElement.nativeElement.focus();
                var /** @type {?} */ fakeEvent = { keyCode: Key.Enter, wasSelection: true, item: item };
                _this.onKeyDown(fakeEvent);
            });
            this.mentionsList.displayTransform = this.getDisplayValue.bind(this);
        }
        this.mentionsList.textAreaElement = this.textAreaInputElement.nativeElement;
        this.mentionsList.show = true;
        this.mentionsList.dropUp = this.dropUp;
        this.mentionsList.activeIndex = 0;
        this.mentionsList.position();
        this.ngZone.run(function () { return _this.mentionsList.resetScroll(); });
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.updateMentionsList = /**
     * @return {?}
     */
    function () {
        var _this = this;
        if (!this.disableSearch) {
            var /** @type {?} */ items = Array.from(this.mentions);
            if (this.searchString) {
                var /** @type {?} */ searchString = this.searchString.toLowerCase(), /** @type {?} */ searchRegEx_1 = new RegExp(escapeRegExp(searchString), 'i');
                items = items.filter(function (item) {
                    var /** @type {?} */ value = _this.getDisplayValue(item);
                    return value !== null && searchRegEx_1.test(value);
                });
                if (this.maxItems > 0) {
                    items = items.slice(0, this.maxItems);
                }
            }
            this.mentionsList.items = items;
        }
        else {
            this.search.emit(this.searchString);
        }
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.parseMarkup = /**
     * @return {?}
     */
    function () {
        if (this.mentionMarkup.length === 0 || this.mentionMarkup[0] !== this.triggerChar) {
            throw new Error("ng-mentions markup pattern must start with the trigger character \"" + this.triggerChar + "\"");
        }
        this.markupSearch = markupToRegExp(this.mentionMarkup);
    };
    /**
     * @param {?=} value
     * @return {?}
     */
    NgMentionsComponent.prototype.parseLines = /**
     * @param {?=} value
     * @return {?}
     */
    function (value) {
        var _this = this;
        if (value === void 0) { value = ''; }
        if (value !== this._value) {
            value = value || '';
            var /** @type {?} */ lines = value.split(this.newLine).map(function (line) { return _this.formatMentions(line); });
            var /** @type {?} */ displayContent = lines.map(function (line) { return line.content; }).join('\n');
            if (this.displayContent !== displayContent) {
                this.lines = lines;
                this.displayContent = displayContent;
                this.triggerChange(value);
            }
        }
    };
    /**
     * @param {?} line
     * @return {?}
     */
    NgMentionsComponent.prototype.formatMentions = /**
     * @param {?} line
     * @return {?}
     */
    function (line) {
        var _this = this;
        var /** @type {?} */ lineObj = /** @type {?} */ ({ originalContent: line, content: line, parts: [] });
        if (line.length === 0) {
            return lineObj;
        }
        var /** @type {?} */ tags = [], /** @type {?} */ mention;
        var /** @type {?} */ regEx = this.markupSearch.regEx;
        regEx.lastIndex = 0;
        while ((mention = regEx.exec(line)) !== null) {
            tags.push({ indices: { start: mention.index, end: mention.index + mention[0].length } });
        }
        var /** @type {?} */ prevTags = [];
        var /** @type {?} */ content = '';
        tags.slice().sort(function (tagA, tagB) { return tagA.indices.start - tagB.indices.start; }).forEach(function (tag) {
            var /** @type {?} */ expectedLength = tag.indices.end - tag.indices.start;
            var /** @type {?} */ contents = line.slice(tag.indices.start, tag.indices.end);
            if (contents.length === expectedLength) {
                var /** @type {?} */ prevIndex = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
                var /** @type {?} */ before = line.slice(prevIndex, tag.indices.start);
                var /** @type {?} */ partMention = /** @type {?} */ ({ contents: contents, tag: tag });
                lineObj.parts.push(before);
                lineObj.parts.push(partMention);
                prevTags.push(tag);
                content += before + _this.formatMention(partMention);
            }
        });
        var /** @type {?} */ remainingStart = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
        var /** @type {?} */ remaining = line.slice(remainingStart);
        lineObj.parts.push(remaining);
        content += remaining;
        lineObj.content = content;
        return lineObj;
    };
    /**
     * @return {?}
     */
    NgMentionsComponent.prototype.refreshStyles = /**
     * @return {?}
     */
    function () {
        var _this = this;
        var /** @type {?} */ element = this.textAreaInputElement.nativeElement;
        var /** @type {?} */ computedStyle = getComputedStyle(element);
        this.highlighterStyle = {};
        styleProperties.forEach(function (prop) {
            _this.highlighterStyle[prop] = computedStyle[prop];
        });
        this.changeDetector.detectChanges();
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NgMentionsComponent.prototype.triggerChange = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this._value = value;
        this.valueChanges.emit(this._value);
        this.changeDetector.detectChanges();
    };
    NgMentionsComponent.decorators = [
        { type: Component, args: [{
                    moduleId: module.id,
                    exportAs: 'ngMentions',
                    selector: 'ng-mentions',
                    template: "\n      <div #highlighter class=\"highlighter\" [ngClass]=\"textAreaClassNames\" [attr.readonly]=\"readonly\"\n           [ngStyle]=\"highlighterStyle\">\n          <div *ngFor=\"let line of lines\">\n              <ng-container *ngFor=\"let part of line.parts\">\n                  <highlighted *ngIf=\"isPartMention(part)\" [tag]=\"part.tag\">{{formatMention(part)}}</highlighted>\n                  <ng-container *ngIf=\"!isPartMention(part)\">{{part}}</ng-container>\n              </ng-container>\n              &nbsp;\n          </div>\n      </div>\n      <textarea\n        #input\n        [rows]=\"rows\"\n        [cols]=\"columns\"\n        [ngModel]=\"displayContent\"\n        [ngClass]=\"textAreaClassNames\"\n        (keydown)=\"onKeyDown($event)\"\n        (blur)=\"onBlur($event)\"\n        (select)=\"onSelect($event)\"\n        (mouseup)=\"onSelect($event)\"\n        (ngModelChange)=\"onChange($event)\"\n        (scroll)=\"onTextAreaScroll()\"\n        [disabled]=\"disabled\"\n        [required]=\"required\"\n        [placeholder]=\"placeholder\"\n      ></textarea>\n  ",
                    styles: [
                        'ng-mentions {position: relative;}',
                        'ng-mentions textarea {position:relative; background-color: transparent !important;}', "ng-mentions .highlighter {\n        position: absolute;\n        top:      0;\n        left:     0;\n        right:    0;\n        bottom:   0;\n        color:    transparent;\n        overflow: hidden !important;\n    }",
                        "ng-mentions highlighted {\n        display:          inline;\n        border-radius:    3px;\n        padding:          1px;\n        margin:           -1px;\n        overflow-wrap:    break-word;\n        background-color: lightblue;\n    }"
                    ],
                    preserveWhitespaces: false,
                    encapsulation: ViewEncapsulation.None
                },] },
    ];
    /** @nocollapse */
    NgMentionsComponent.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: ComponentFactoryResolver, },
        { type: ViewContainerRef, },
        { type: ChangeDetectorRef, },
        { type: NgZone, },
    ]; };
    NgMentionsComponent.propDecorators = {
        "triggerChar": [{ type: Input, args: ['triggerChar',] },],
        "mentionMarkup": [{ type: Input, args: ['markup',] },],
        "disableSearch": [{ type: Input, args: ['disableSearch',] },],
        "maxItems": [{ type: Input, args: ['maxItems',] },],
        "dropUp": [{ type: Input, args: ['dropUp',] },],
        "displayName": [{ type: Input, args: ['displayName',] },],
        "formClass": [{ type: Input, args: ['formClass',] },],
        "placeholder": [{ type: Input, args: ['placeholder',] },],
        "value": [{ type: Input, args: ['value',] },],
        "required": [{ type: Input, args: ['required',] },],
        "disabled": [{ type: Input, args: ['disabled',] },],
        "rows": [{ type: Input, args: ['rows',] },],
        "columns": [{ type: Input, args: ['cols',] },],
        "mentionItems": [{ type: Input, args: ['mentions',] },],
        "search": [{ type: Output, args: ['search',] },],
        "valueChanges": [{ type: Output, args: ['valueChanges',] },],
        "stateChanges": [{ type: Output, args: ['stateChanges',] },],
        "mentionListTemplate": [{ type: ContentChild, args: [TemplateRef,] },],
        "textAreaInputElement": [{ type: ViewChild, args: ['input',] },],
        "highlighterElement": [{ type: ViewChild, args: ['highlighter',] },],
        "onWindowResize": [{ type: HostListener, args: ['window:resize',] },],
    };
    return NgMentionsComponent;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * The MentionsAccessorDirective directive is used to indicate the input element.
 * This directive is required.
 */
var NgMentionsAccessorDirective = /** @class */ (function () {
    function NgMentionsAccessorDirective(element, host) {
        this.element = element;
        this.host = host;
        this._destroyed = new Subject();
    }
    /**
     * @return {?}
     */
    NgMentionsAccessorDirective.prototype.ngOnInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.host.valueChanges.pipe(takeUntil(this._destroyed)).subscribe(function (value) { return _this.onChange(value); });
    };
    /**
     * @return {?}
     */
    NgMentionsAccessorDirective.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        this._destroyed.next();
        this._destroyed.complete();
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NgMentionsAccessorDirective.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NgMentionsAccessorDirective.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onTouch = fn;
    };
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    NgMentionsAccessorDirective.prototype.setDisabledState = /**
     * @param {?} isDisabled
     * @return {?}
     */
    function (isDisabled) {
        this.host.disabled = isDisabled;
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NgMentionsAccessorDirective.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (typeof value === 'string' || value === null) {
            this.host.value = value;
        }
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NgMentionsAccessorDirective.prototype.onChange = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (this._onChange) {
            this._onChange(value);
        }
    };
    /**
     * @return {?}
     */
    NgMentionsAccessorDirective.prototype.onTouched = /**
     * @return {?}
     */
    function () {
        if (this._onTouch) {
            this._onTouch();
        }
    };
    NgMentionsAccessorDirective.decorators = [
        { type: Directive, args: [{
                    exportAs: 'ngMentions',
                    selector: 'ng-mentions',
                    host: { '(change)': 'onChange($event)', '(touch)': 'onTouched()' },
                    providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return NgMentionsAccessorDirective; }), multi: true }]
                },] },
    ];
    /** @nocollapse */
    NgMentionsAccessorDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: NgMentionsComponent, },
    ]; };
    return NgMentionsAccessorDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var HighlightedDirective = /** @class */ (function () {
    function HighlightedDirective() {
    }
    HighlightedDirective.decorators = [
        { type: Directive, args: [{ selector: 'highlighted' },] },
    ];
    /** @nocollapse */
    HighlightedDirective.ctorParameters = function () { return []; };
    HighlightedDirective.propDecorators = {
        "tag": [{ type: Input },],
    };
    return HighlightedDirective;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
var EXPORT_DIRECTIVES = [NgMentionsComponent, NgMentionsAccessorDirective];
var DECLARATIONS = [NgMentionsComponent, NgMentionsAccessorDirective, NgMentionsListComponent, HighlightedDirective];
var NgMentionsModule = /** @class */ (function () {
    function NgMentionsModule() {
    }
    NgMentionsModule.decorators = [
        { type: NgModule, args: [{
                    imports: [CommonModule, FormsModule],
                    exports: EXPORT_DIRECTIVES,
                    declarations: DECLARATIONS,
                    entryComponents: [NgMentionsListComponent]
                },] },
    ];
    /** @nocollapse */
    NgMentionsModule.ctorParameters = function () { return []; };
    return NgMentionsModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { NgMentionsAccessorDirective, NgMentionsComponent, NgMentionsModule, HighlightedDirective as ɵb, NgMentionsListComponent as ɵa };
//# sourceMappingURL=ng-mentions.js.map
